@@ -85,9 +85,9 @@
 database_type='Psycopg'
 __doc__='''%s Database Connection
 
-$Id: DA.py,v 1.20.2.11 2004/05/21 08:33:37 fog Exp $''' % database_type
-__version__='$Revision: 1.20.2.11 $'[11:-2]
-__psycopg_versions__ = ('1.1.12',)
+$Id: DA.py 618 2005-01-05 20:05:54Z fog $''' % database_type
+__version__='$Revision: 1.20.2.14 $'[11:-2]
+__psycopg_versions__ = ('1.1.17','1.1.18')
 
 
 from db import DB
@@ -97,7 +97,8 @@ from ExtensionClass import Base
 from string import find, join, split, rindex
 try:
     import psycopg
-    from psycopg import new_type, register_type, DATETIME, TIME, DATE, INTERVAL
+    from psycopg import new_type, register_type, \
+         DATETIME, TIME, DATE, INTERVAL, STRING
 except StandardError, err:
     print err
 try:
@@ -108,6 +109,7 @@ try:
     from App.Dialogs import MessageDialog
 except:
     pass
+import time
 
 manage_addZPsycopgConnectionForm = HTMLFile('connectionAdd', globals())
 def manage_addZPsycopgConnection(self, id, title,
@@ -119,7 +121,7 @@ def manage_addZPsycopgConnection(self, id, title,
     if REQUEST is not None: return self.manage_main(self,REQUEST)
 
 
-# Convert an ISO timestamp string from postgres to a DateTime(zope version)
+# Convert an ISO timestamp string from postgres to a DateTime (zope version)
 # object.
 def cast_DateTime(str):
     if str:
@@ -129,9 +131,10 @@ def cast_DateTime(str):
             # we now should split out any timezone info
             dt[1] = split(dt[1], '-')[0]
             dt[1] = split(dt[1], '+')[0]
-            return DateTime(join(dt[:2], ' '))
+            s = split(join(dt[:2], ' '), '.')[0]
         else:
-            return DateTime(dt[0])
+            s = dt[0]
+        return DateTime(s)
 
 # Convert an ISO date string from postgres to a DateTime(zope version)
 # object.
@@ -154,6 +157,14 @@ def cast_Time(str):
 def cast_Interval(str):
     return str
 
+# Convert a string to Unicode. Factory function as we depend on the encoding.
+def cast_String_factory(encoding):
+    def cast_String(s):
+        if s is not None:
+            return unicode(s, encoding)
+        else:
+            return s
+    return cast_String
 
 class Connection(DABase.Connection):
     "The connection class."
@@ -163,28 +174,35 @@ class Connection(DABase.Connection):
     icon = 'misc_/Z%sDA/conn' % database_type
 
     def __init__(self, id, title, connection_string, zdatetime,
-                 check=None, tilevel=2):
+                 check=None, tilevel=2, encoding='UTF-8', ustrings=0):
         self.zdatetime=zdatetime
         self.id=str(id)
         self.edit(title, connection_string, zdatetime,
-                  check=check, tilevel=tilevel)
+                  check=check, tilevel=tilevel, encoding=encoding,
+                  ustrings=ustrings)
 
-    def edit(self, title, connection_string, zdatetime, check=1, tilevel=2):
+    def edit(self, title, connection_string, zdatetime,
+             check=1, tilevel=2, encoding='UTF-8', ustrings=0):
         self.title=title
         self.connection_string=connection_string
         self.zdatetime=zdatetime
 	self.tilevel=tilevel
+        self.encoding=encoding
+        self.ustrings=ustrings
         self.set_type_casts()
         if check: self.connect(connection_string)
 
     manage_properties=HTMLFile('connectionEdit', globals())
 
     def manage_edit(self, title, connection_string,
-                    zdatetime=None, check=None, tilevel=2, REQUEST=None):
+                    zdatetime=None, check=None, tilevel=2, encoding='UTF-8',
+                    ustrings=0,
+                    REQUEST=None):
         """Change connection
         """
         self.edit(title, connection_string, zdatetime,
-                  check=check, tilevel=tilevel)
+                  check=check, tilevel=tilevel, encoding=encoding,
+                  ustrings=ustrings)
         if REQUEST is not None:
             return MessageDialog(
                 title='Edited',
@@ -210,6 +228,12 @@ class Connection(DABase.Connection):
             register_type(DATE)
             register_type(TIME)
             register_type(INTERVAL)
+        if getattr(self, "ustrings", 0) and self.encoding:
+            USTRING = psycopg.new_type(psycopg.STRING.values, "USTRING",
+                                       cast_String_factory(self.encoding))
+            register_type(USTRING)
+        else:
+            register_type(STRING)
             
     def factory(self):
         return DB
@@ -235,7 +259,11 @@ class Connection(DABase.Connection):
                 # having to recreate the connection object
                 if not hasattr(self, 'tilevel'):
                     self.tilevel = 2
-                self._v_database_connection=DB(s, self.tilevel)
+                if not hasattr(self, 'encoding'):
+                    self.encoding = 'UTF-8'
+                if not hasattr(self, 'ustrings'):
+                    self.ustrings = 0
+                self._v_database_connection=DB(s, self.tilevel, self.encoding)
             except:
                 t, v, tb = sys.exc_info()
                 raise 'BadRequest', (
