@@ -70,7 +70,7 @@
 #   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 #   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 #   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
-#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+#   USE, DATA, OR PROFITS; OR BUSINESS INTERUPTION) HOWEVER CAUSED AND
 #   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 #   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 #   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
@@ -83,8 +83,8 @@
 # 
 ##############################################################################
 
-'''$Id: db.py,v 1.31.2.6 2004/05/16 08:09:50 fog Exp $'''
-__version__='$Revision: 1.31.2.6 $'[11:-2]
+'''$Id: db.py 554 2004-10-30 00:19:27Z fog $'''
+__version__='$Revision: 1.31.2.7 $'[11:-2]
 
 
 from Shared.DC.ZRDB.TM import TM
@@ -105,9 +105,10 @@ class DB(TM,dbi_db.DB):
 
     _p_oid = _p_changed = _registered = None
 
-    def __init__(self, connection, tilevel):
+    def __init__(self, connection, tilevel, enc='utf-8'):
         self.connection = connection
         self.tilevel = tilevel
+        self.encoding = enc 
         self.db = self.connect(self.connection)
         self.failures = 0
         self.calls = 0
@@ -186,7 +187,7 @@ class DB(TM,dbi_db.DB):
                 'Nullable': 0})
         return r
 
-    def query(self, query_string, max_rows=None):
+    def query(self, query_string, max_rows=None, query_data=None):
         self._register()
         self.calls = self.calls+1
         
@@ -198,8 +199,14 @@ class DB(TM,dbi_db.DB):
         
         try:
             for qs in filter(None, map(strip, split(query_string, '\0'))):
+                if type(qs) == unicode:
+                    if self.encoding:
+                        qs = qs.encode(self.encoding)
                 try:
-                    r = c.execute(qs)
+                    if (query_data):
+                        r = c.execute(qs, query_data)
+                    else:
+                        r = c.execute(qs)
                 except (psycopg.ProgrammingError,psycopg.IntegrityError), perr:
                     if perr.args[0].find("concurrent update") > -1:
                         raise ConflictError
