@@ -21,7 +21,7 @@
 
 /*
  * cursor.c - defines the cursor object DBAPI 2.0
- * $Id: cursor.c,v 1.109.2.24 2004/07/29 21:52:38 fog Exp $
+ * $Id: cursor.c 577 2004-11-17 11:42:24Z fog $
  */
 
 #ifdef _WIN32
@@ -776,9 +776,16 @@ _psyco_curs_execute(cursobject *self, char *query,
         Dprintf("_psyco_curs_execute: command returned COPY_IN\n");
         
         /* call the callback */
-        res = cb(self, cb_args);
-        if (PyErr_Occurred()) {
-            Py_XDECREF(res);
+        if (cb && cb_args) {
+            res = cb(self, cb_args);
+            if (PyErr_Occurred()) {
+                Py_XDECREF(res);
+                goto error;
+            }
+        }
+        else {
+            PyErr_SetString(ProgrammingError,
+                            "COPY TO/COPY FROM can't be used in .execute()");
             goto error;
         }
         break;
@@ -921,7 +928,7 @@ _psyco_curs_execute(cursobject *self, char *query,
     case PGRES_COMMAND_OK:
         Dprintf("_psyco_curs_execute: command returned OK (no tuples)\n");
         /* sets rowcount to the right number of affected tuples */
-        self->rowcount = PQntuples(self->pgres);
+        self->rowcount = atol(PQcmdTuples(self->pgres));
         /* try to obtain the oid of an insert */
         self->last_oid = PQoidValue(self->pgres);
 
@@ -996,7 +1003,7 @@ _mogrify(PyObject *var, PyObject *fmt, PyObject **new)
 {
     PyObject *key, *value, *n, *item;
     char *d, *c;
-    int index = 0;
+    int index = 0, force = 0;
 
     /* from now on we'll use n and replace its value in *new only at the
        end, just before returning */
@@ -1006,7 +1013,7 @@ _mogrify(PyObject *var, PyObject *fmt, PyObject **new)
     while(*c) {
         /* handle plain percent symbol in format string */
         if (c[0] == '%' && c[1] == '%') {
-            c+=2;
+            c+=2; force = 1;
         }
         
         /* if we find '%(' then this is a dictionary */
@@ -1072,9 +1079,11 @@ _mogrify(PyObject *var, PyObject *fmt, PyObject **new)
                     if (*d) *d = 's';
                 }
                 Py_DECREF(key); /* key has the original refcnt now */
-                Dprintf("_mogrify: after value refcnt: %d\n", value->ob_refcnt);
+                Dprintf("_mogrify: after value refcnt: %d\n",
+                        value->ob_refcnt);
             }
             c = d;
+
         }
         else if (c[0] == '%' && c[1] != '(') {
             value = PySequence_GetItem(var, index);
@@ -1119,7 +1128,10 @@ _mogrify(PyObject *var, PyObject *fmt, PyObject **new)
         }
     }
 
+    if (force && n == NULL)
+        n = PyTuple_New(0);
     *new = n;
+    
     return 0;
 }
 
